/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

/******************************* C H E C K *************************************/

static check_where_construct_shapes (where)
Node *where;
{
	struct S_WhereConstruct *wc = &where->WhereConstruct;
	int rank_mask, rank_stmt;
	Node *sta, *temp;

	if (wc->WhereConstructStmt == NN)
		return;

	rank_mask = RANK_FROM_SHAPE (wc->WhereConstructStmt->WhereConstructStmt.Expr->com.typ->shape);

	for (LIST_LOOP (sta, wc->WhereBlock, temp)) {
		rank_stmt = RANK_FROM_SHAPE (sta->Statement.Stmt->com.typ->shape);

		if (rank_mask != rank_stmt) {
			sprintf (serr
			, "assignement stmt is of rank %d, must be of rank %d"
			, rank_stmt, rank_mask);
			fatal_fl (serr, sta->Statement.file, sta->Statement.line);
		}
	}
	
	for (LIST_LOOP (sta, wc->ElsewhereBlock, temp)) {
		rank_stmt = RANK_FROM_SHAPE (sta->Statement.Stmt->com.typ->shape);

		if (rank_mask != rank_stmt) {
			sprintf (serr
			, "assignement stmt is of rank %d, must be of rank %d"
			, rank_stmt, rank_mask);
			fatal_fl (serr, sta->Statement.file, sta->Statement.line);
		}
	}
}

Node *check_WhereStmt (root)
Node *root;
{
	Node *expr = root->WhereStmt.Expr;
	Node *stmt = root->WhereStmt.AssignementStmt;
	int rank_mask, rank_stmt;

	if (expr->com.typ->etype->type != TYP_LOGICAL) {
		sprintf (serr
		, "type of mask expression in WHERE statement is %s, must be LOGICAL"
		, etype_name (expr->com.typ->etype));
		fatal (serr);  /* E_7_21_45 */
	}

/* ZEKI I make this part of ; because it comes FORALL instead of
   assignement statement 
   so here it will not detect if threre is mismatch expr shape with stmt shape


	rank_mask = RANK_FROM_SHAPE (expr->com.typ->shape);
	rank_stmt = RANK_FROM_SHAPE (stmt->com.typ->shape);

	if (rank_mask != rank_stmt) {
		sprintf (serr
		, "assignement stmt is of rank %d, must be of rank %d"
		, rank_stmt, rank_mask);
		fatal (serr);
	}

	root->com.typ = typ_expr (expr, &Eq, stmt);
*/
        root=where_to_forall(root);
	return root;
}

Node *check_WhereConstructStmt (root)
Node *root;
{
	Node *expr = root->WhereConstructStmt.Expr;
	Node *where;
	Node *label;

	if (expr->com.typ->etype->type != TYP_LOGICAL) {
		sprintf (serr
		, "type of mask expression in WHERE construct statement is %s, must be LOGICAL"
		, etype_name (expr->com.typ->etype));
		fatal (serr);  /* E_7_21_45 */
	}

	where = make (WHERE_CONSTRUCT);
	block_push (where);
	BLOCK_ADD_STMTS_TO (where->WhereConstruct.WhereBlock);
	where->com.typ = root->com.typ = expr->com.typ;
	where->WhereConstruct.WhereConstructStmt = root;

    label = where->WhereConstruct.label = se_current_statement->Statement.IntegerConstant;
    where->WhereConstruct.file = input_filename;
    where->WhereConstruct.line = zzlineno;

	IGNORE_RETURN;
}

Node *check_ElsewhereStmt (root)
Node *root;
{
	Node *where;

	if ((where = block_match ("ELSE WHERE", WHERE_CONSTRUCT)) != NN) {
		BLOCK_ADD_STMTS_TO(where->WhereConstruct.ElsewhereBlock);
		where->WhereConstruct.ElsewhereStmt = root;
	}

	IGNORE_RETURN;
}

Node *check_EndWhereStmt (root)
Node *root;
{
	Node *where;

	if ((where = block_match ("END WHERE", WHERE_CONSTRUCT)) == NN)
		IGNORE_RETURN;

	where = block_pop ();
	where->WhereConstruct.EndWhereStmt = root;
	where->WhereConstruct.WhereConstructStmt->WhereConstructStmt.Expr
	= temp_replace (where->WhereConstruct.WhereConstructStmt->WhereConstructStmt.Expr); /*!!! */

    where->WhereConstruct.label_endwhere = se_current_statement->Statement.IntegerConstant;

	check_where_construct_shapes (where);

	block_add_stmt_with_label (where, where->WhereConstruct.label, where->WhereConstruct.file, where->WhereConstruct.line);
	where->WhereConstruct.trans = TRN;

	IGNORE_RETURN;
}

/***************************** T R A N S L A T E **********************************/

Node *trans_WhereStmt (node)
Node *node;
{
/* ##BEG##, 10/30/1991, ZEKI */
/* The following comment eliminate to crate the temp variable for where condition */
/*	node->WhereStmt.Expr = temp_replace (node->WhereStmt.Expr);*/
	node->WhereStmt.Where = &If;
	return node;
}

/******************************** W R I T E ***************************************/

int se_in_where = 0;

write_WhereStmt (node)
Node *node;
{
	se_in_where = 1;

	if (node->com.typ->shape != (Shape*) 0) {
		array_dos (node->com.typ);
		otab ();
		write_pass_childs (node);
		odon ();
		array_enddos (node->com.typ);
	}
	else {
		write_pass_childs (node);
	}

	se_in_where = 0;
}

write_WhereConstruct (node)
Node *node;
{
	struct S_WhereConstruct *where = &node->WhereConstruct;
	Node *sta, *temp;

	sta_write_sta_bef (where->WhereConstructStmt->WhereConstructStmt.Expr);

	where->WhereBlock = list_invert (where->WhereBlock);
	where->ElsewhereBlock = list_invert (where->ElsewhereBlock);

	for (LIST_LOOP (sta, where->WhereBlock, temp))
		where_write_sta (sta, node, where, 0);
	for (LIST_LOOP (sta, where->ElsewhereBlock, temp))
		where_write_sta (sta, node, where, 1);

	sta_write_sta_aft (where->WhereConstructStmt->WhereConstructStmt.Expr);

    if (where->label_endwhere != NN)
        o_continue (where->label_endwhere);
}

/* ##BEG##,10/12/91, ZEKI */
where_write_sta (sta, node, where, is_else)
Node *sta, *node;
struct S_WhereConstruct *where;
int is_else;
{
	sta_write_sta_bef (sta);

	se_in_where = 1;
	oTab ();
	array_dos (node->com.typ);
	otab ();
	oIF ();
        ochb ('(');
	if (is_else) ostr(".NOT.");
	write_pass (tree_dup (where->WhereConstructStmt->WhereConstructStmt.Expr));
	ochb (')');
        write_pass ( Node_Then);
        oDon ();
	write_pass (sta);
        
        oendif ();
      
        ostart ();
	array_enddos (node->com.typ);
	oDon ();


	se_in_where = 0;
}


array_dos_npac (typ)  /* writes Array FORALLs from typ */
Typ *typ;
{
	int i;
	Shape *shape;

	if (typ == (Typ*) 0)
		return;
	shape = typ->shape;

	for (i = 0; i < shape->rank; i++) {
		if (i) {
			otab ();
		}
		else {
                	oFORALL ();
                        ostr ("(");
		      }
		array_loop (i, shape);
                if (i != ((shape->rank)-1)) ostr(",");
	}
}

/* ##END##,10/12/91, ZEKI */
